@@ -9,7 +9,8 @@ import yaml
 from conans.client.file_copier import FileCopier
 from conans.client.output import Color, ScopedOutput
 from conans.client.remover import DiskRemover
-from conans.errors import ConanException
+from conans.client.tools import chdir
+from conans.errors import ConanException, ConanV2Exception, conanfile_exception_formatter
 from conans.model.manifest import FileTreeManifest
 from conans.model.ref import ConanFileReference
 from conans.model.scm import SCM, get_scm_data
@@ -21,6 +22,7 @@ from conans.util.log import logger
 
 isPY38 = bool(sys.version_info.major == 3 and sys.version_info.minor == 8)
 
+
 def export_alias(package_layout, target_ref, output, revisions_enabled):
     revision_mode = "hash"
     conanfile = """
@@ -69,11 +71,15 @@ def cmd_export(app, conanfile_path, name, version, user, channel, keep_source,
     # Take the default from the env vars if they exist to not break behavior
     try:
         user = conanfile.user
+    except ConanV2Exception:
+        raise
     except ConanException:
         user = None
 
     try:
         channel = conanfile.channel
+    except ConanV2Exception:
+        raise
     except ConanException:
         channel = None
 
@@ -81,7 +87,7 @@ def cmd_export(app, conanfile_path, name, version, user, channel, keep_source,
 
     # If we receive lock information, python_requires could have been locked
     if graph_lock:
-        node_id = graph_lock.get_node(ref)
+        node_id = graph_lock.get_consumer(ref)
         python_requires = graph_lock.python_requires(node_id)
         # TODO: check that the locked python_requires are different from the loaded ones
         app.range_resolver.clear_output()  # invalidate previous version range output
@@ -222,7 +228,8 @@ def _check_settings_for_warnings(conanfile, output):
         pass
 
 
-def _capture_scm_auto_fields(conanfile, conanfile_dir, package_layout, output, ignore_dirty, scm_to_conandata):
+def _capture_scm_auto_fields(conanfile, conanfile_dir, package_layout, output, ignore_dirty,
+                             scm_to_conandata):
     """Deduce the values for the scm auto fields or functions assigned to 'url' or 'revision'
        and replace the conanfile.py contents.
        Returns a tuple with (scm_data, path_to_scm_local_directory)"""
@@ -251,7 +258,11 @@ def _capture_scm_auto_fields(conanfile, conanfile_dir, package_layout, output, i
     if scm_data.url == "auto":
         origin = scm.get_qualified_remote_url(remove_credentials=True)
         if not origin:
-            raise ConanException("Repo origin cannot be deduced")
+            output.warn("Repo origin cannot be deduced, 'auto' fields won't be replaced."
+                        " 'conan upload' command will prevent uploading recipes with 'auto'"
+                        " values in these fields.")
+            local_src_path = scm.get_local_path_to_url(origin)
+            return scm_data, local_src_path
         if scm.is_local_repository():
             output.warn("Repo origin looks like a local path: %s" % origin)
         output.success("Repo origin deduced by 'auto': %s" % origin)
@@ -276,7 +287,8 @@ def _replace_scm_data_in_recipe(package_layout, scm_data, scm_to_conandata):
         if os.path.exists(conandata_path):
             conandata_yml = yaml.safe_load(load(conandata_path))
             if '.conan' in conandata_yml:
-                raise ConanException("Field '.conan' inside '{}' file is reserved to Conan usage.".format(DATA_YML))
+                raise ConanException("Field '.conan' inside '{}' file is reserved to "
+                                     "Conan usage.".format(DATA_YML))
         scm_data_copied = scm_data.as_dict()
         scm_data_copied.pop('username', None)
         scm_data_copied.pop('password', None)
@@ -288,6 +300,7 @@ def _replace_scm_data_in_recipe(package_layout, scm_data, scm_to_conandata):
 
 
 def _replace_scm_data_in_conanfile(conanfile_path, scm_data):
+    # FIXME: Remove in Conan 2.0, it will use conandata.yml as the only way
     # Parsing and replacing the SCM field
     content = load(conanfile_path)
     headers = []
@@ -328,8 +341,9 @@ def _replace_scm_data_in_conanfile(conanfile_path, scm_data):
                                 # Next statement can be a comment or anything else
                                 next_statement = statements[i+1]
                                 if isPY38 and isinstance(next_statement, ast.Expr):
-                                    # Python 3.8 properly parses multiline comments with start and end lines,
-                                    #  here we preserve the same (wrong) implementation of previous releases
+                                    # Python 3.8 properly parses multiline comments with start
+                                    # and end lines, here we preserve the same (wrong)
+                                    # implementation of previous releases
                                     next_line = next_statement.end_lineno - 1
                                 else:
                                     next_line = next_statement.lineno - 1
@@ -442,6 +456,10 @@ def _export_scm(scm_data, origin_folder, scm_sources_folder, output):
 
 
 def export_source(conanfile, origin_folder, destination_source_folder):
+    if callable(conanfile.exports_sources):
+        raise ConanException("conanfile 'exports_sources' shouldn't be a method, "
+                             "use 'export_sources()' instead")
+
     if isinstance(conanfile.exports_sources, str):
         conanfile.exports_sources = (conanfile.exports_sources, )
 
@@ -453,8 +471,12 @@ def export_source(conanfile, origin_folder, destination_source_folder):
     package_output = ScopedOutput("%s exports_sources" % output.scope, output)
     copier.report(package_output)
 
+    _run_method(conanfile, "export_sources", origin_folder, destination_source_folder, output)
+
 
 def export_recipe(conanfile, origin_folder, destination_folder):
+    if callable(conanfile.exports):
+        raise ConanException("conanfile 'exports' shouldn't be a method, use 'export()' instead")
     if isinstance(conanfile.exports, str):
         conanfile.exports = (conanfile.exports, )
 
@@ -478,5 +500,30 @@ def export_recipe(conanfile, origin_folder, destination_folder):
     copier = FileCopier([origin_folder], destination_folder)
     for pattern in included_exports:
         copier(pattern, links=True, excludes=excluded_exports)
-
     copier.report(package_output)
+
+    _run_method(conanfile, "export", origin_folder, destination_folder, output)
+
+
+def _run_method(conanfile, method, origin_folder, destination_folder, output):
+    export_method = getattr(conanfile, method, None)
+    if export_method:
+        if not callable(export_method):
+            raise ConanException("conanfile '%s' must be a method" % method)
+        output.highlight("Calling %s()" % method)
+        copier = FileCopier([origin_folder], destination_folder)
+        conanfile.copy = copier
+        folder_name = "%s_folder" % method
+        setattr(conanfile, folder_name, destination_folder)
+        default_options = conanfile.default_options
+        try:
+            # TODO: Poor man attribute control access. Convert to nice decorator
+            conanfile.default_options = None
+            with chdir(origin_folder):
+                with conanfile_exception_formatter(str(conanfile), method):
+                    export_method()
+        finally:
+            conanfile.default_options = default_options
+            delattr(conanfile, folder_name)
+        export_method_output = ScopedOutput("%s %s() method" % (output.scope, method), output)
+        copier.report(export_method_output)
