@@ -4,11 +4,12 @@ from conans.client.graph.build_mode import BuildMode
 from conans.client.graph.graph import (BINARY_BUILD, BINARY_CACHE, BINARY_DOWNLOAD, BINARY_MISSING,
                                        BINARY_UPDATE, RECIPE_EDITABLE, BINARY_EDITABLE,
                                        RECIPE_CONSUMER, RECIPE_VIRTUAL, BINARY_SKIP, BINARY_UNKNOWN)
-
-from conans.errors import NoRemoteAvailable, NotFoundException, conanfile_exception_formatter
+from conans.errors import NoRemoteAvailable, NotFoundException, conanfile_exception_formatter, \
+    ConanException
 from conans.model.info import ConanInfo, PACKAGE_ID_UNKNOWN
 from conans.model.manifest import FileTreeManifest
 from conans.model.ref import PackageReference
+from conans.util.conan_v2_mode import conan_v2_property
 from conans.util.files import is_dirty, rmdir
 
 
@@ -20,14 +21,14 @@ class GraphBinariesAnalyzer(object):
         self._remote_manager = remote_manager
         # These are the nodes with pref (not including PREV) that have been evaluated
         self._evaluated = {}  # {pref: [nodes]}
+        self._fixed_package_id = cache.config.full_transitive_package_id
 
     @staticmethod
-    def _check_update(upstream_manifest, package_folder, output, node):
+    def _check_update(upstream_manifest, package_folder, output):
         read_manifest = FileTreeManifest.load(package_folder)
         if upstream_manifest != read_manifest:
             if upstream_manifest.time > read_manifest.time:
                 output.warn("Current package is older than remote upstream one")
-                node.update_manifest = upstream_manifest
                 return True
             else:
                 output.warn("Current package is newer than remote upstream one")
@@ -83,7 +84,7 @@ class GraphBinariesAnalyzer(object):
                 except NoRemoteAvailable:
                     output.warn("Can't update, no remote defined")
                 else:
-                    if self._check_update(upstream_manifest, package_folder, output, node):
+                    if self._check_update(upstream_manifest, package_folder, output):
                         node.binary = BINARY_UPDATE
                         node.prev = pref.revision  # With revision
             elif remotes:
@@ -156,13 +157,22 @@ class GraphBinariesAnalyzer(object):
 
         # If it has lock
         locked = node.graph_lock_node
-        if locked and locked.pref.id == node.package_id:
-            pref = locked.pref  # Keep the locked with PREV
+        if locked and locked.package_id:  # if package_id = None, nothing to lock here
+            # First we update the package_id, just in case there are differences or something
+            if locked.package_id != node.package_id and locked.package_id != PACKAGE_ID_UNKNOWN:
+                raise ConanException("'%s' package-id '%s' doesn't match the locked one '%s'"
+                                     % (repr(locked.ref), node.package_id, locked.package_id))
+            locked.package_id = node.package_id  # necessary for PACKAGE_ID_UNKNOWN
+            pref = PackageReference(locked.ref, locked.package_id, locked.prev)  # Keep locked PREV
             self._process_node(node, pref, build_mode, update, remotes)
             if node.binary == BINARY_MISSING and build_mode.allowed(node.conanfile):
                 node.binary = BINARY_BUILD
+            if node.binary == BINARY_BUILD and locked.prev:
+                raise ConanException("Cannot build '%s' because it is already locked in the input "
+                                     "lockfile" % repr(node.ref))
         else:
             assert node.prev is None, "Non locked node shouldn't have PREV in evaluate_node"
+            assert node.binary is None, "Node.binary should be None if not locked"
             pref = PackageReference(node.ref, node.package_id)
             self._process_node(node, pref, build_mode, update, remotes)
             if node.binary == BINARY_MISSING:
@@ -183,6 +193,7 @@ class GraphBinariesAnalyzer(object):
                             node.conanfile.output.info("Main binary package '%s' missing. Using "
                                                        "compatible package '%s'"
                                                        % (node.package_id, package_id))
+                            # Modifying package id under the hood, FIXME
                             node._package_id = package_id
                             # So they are available in package_info() method
                             node.conanfile.settings.values = compatible_package.settings
@@ -191,6 +202,10 @@ class GraphBinariesAnalyzer(object):
                 if node.binary == BINARY_MISSING and build_mode.allowed(node.conanfile):
                     node.binary = BINARY_BUILD
 
+            if locked:
+                locked.package_id = node.package_id
+                locked.prev = node.prev
+
     def _process_node(self, node, pref, build_mode, update, remotes):
         # Check that this same reference hasn't already been checked
         if self._evaluate_is_cached(node, pref):
@@ -264,7 +279,24 @@ class GraphBinariesAnalyzer(object):
         conanfile.options.freeze()
 
     @staticmethod
-    def _compute_package_id(node, default_package_id_mode, default_python_requires_id_mode):
+    def package_id_transitive_reqs(node):
+        """
+        accumulate the direct and transitive requirements prefs necessary to compute the
+        package_id
+        :return: set(prefs) of direct deps, set(prefs) of transitive deps
+        """
+        node.id_direct_prefs = set()  # of PackageReference
+        node.id_indirect_prefs = set()  # of PackageReference, avoid duplicates
+        neighbors = [d.dst for d in node.dependencies if not d.build_require]
+        for neighbor in neighbors:
+            node.id_direct_prefs.add(neighbor.pref)
+            node.id_indirect_prefs.update(neighbor.id_direct_prefs)
+            node.id_indirect_prefs.update(neighbor.id_indirect_prefs)
+        # Make sure not duplicated, totally necessary
+        node.id_indirect_prefs.difference_update(node.id_direct_prefs)
+        return node.id_direct_prefs, node.id_indirect_prefs
+
+    def _compute_package_id(self, node, default_package_id_mode, default_python_requires_id_mode):
         """
         Compute the binary package ID of this node
         :param node: the node to compute the package-ID
@@ -274,15 +306,18 @@ class GraphBinariesAnalyzer(object):
         # A bit risky to be done now
         conanfile = node.conanfile
         neighbors = node.neighbors()
-        direct_reqs = []  # of PackageReference
-        indirect_reqs = set()  # of PackageReference, avoid duplicates
-        for neighbor in neighbors:
-            ref, nconan = neighbor.ref, neighbor.conanfile
-            direct_reqs.append(neighbor.pref)
-            indirect_reqs.update(nconan.info.requires.refs())
 
-        # Make sure not duplicated
-        indirect_reqs.difference_update(direct_reqs)
+        direct_reqs, indirect_reqs = self.package_id_transitive_reqs(node)
+
+        # FIXME: Conan v2.0 This is introducing a bug for backwards compatibility, it will add
+        #   only the requirements available in the 'neighbour.info' object, not all the closure
+        if not self._fixed_package_id:
+            old_indirect = set()
+            for neighbor in neighbors:
+                old_indirect.update((p.ref, p.id) for p in neighbor.conanfile.info.requires.refs())
+            indirect_reqs = set(p for p in indirect_reqs if (p.ref, p.id) in old_indirect)
+            indirect_reqs.difference_update(direct_reqs)
+
         python_requires = getattr(conanfile, "python_requires", None)
         if python_requires:
             if isinstance(python_requires, dict):
@@ -300,7 +335,9 @@ class GraphBinariesAnalyzer(object):
 
         # Once we are done, call package_id() to narrow and change possible values
         with conanfile_exception_formatter(str(conanfile), "package_id"):
-            conanfile.package_id()
+            with conan_v2_property(conanfile, 'cpp_info',
+                                   "'self.cpp_info' access in package_id() method is deprecated"):
+                conanfile.package_id()
 
         info = conanfile.info
         node.package_id = info.package_id()
@@ -322,6 +359,9 @@ class GraphBinariesAnalyzer(object):
         deps_graph.mark_private_skippable(nodes_subset=nodes_subset, root=root)
 
     def reevaluate_node(self, node, remotes, build_mode, update):
+        """ reevaluate the node is necessary when there is some PACKAGE_ID_UNKNOWN due to
+        package_revision_mode
+        """
         assert node.binary == BINARY_UNKNOWN
         output = node.conanfile.output
         node._package_id = None  # Invalidate it, so it can be re-computed
