@@ -1,7 +1,7 @@
 import time
-from collections import OrderedDict
 
-from conans.client.graph.graph import DepsGraph, Node, RECIPE_EDITABLE
+from conans.client.conanfile.configure import run_configure_method
+from conans.client.graph.graph import DepsGraph, Node, RECIPE_EDITABLE, CONTEXT_HOST, CONTEXT_BUILD
 from conans.errors import (ConanException, ConanExceptionInUserConanfileMethod,
                            conanfile_exception_formatter)
 from conans.model.conan_file import get_env_context_manager
@@ -47,27 +47,29 @@ class DepsGraphBuilder(object):
         self._resolver = resolver
         self._recorder = recorder
 
-    def load_graph(self, root_node, check_updates, update, remotes, profile_host, graph_lock=None):
+    def load_graph(self, root_node, check_updates, update, remotes, profile_host, profile_build,
+                   graph_lock=None):
         check_updates = check_updates or update
         initial = graph_lock.initial_counter if graph_lock else None
         dep_graph = DepsGraph(initial_node_id=initial)
         # compute the conanfile entry point for this dependency graph
-        name = root_node.name
-        root_node.public_closure = OrderedDict([(name, root_node)])
-        root_node.transitive_closure = OrderedDict([(name, root_node)])
-        root_node.public_deps = {name: root_node}
-        root_node.ancestors = set()
+        root_node.public_closure.add(root_node)
+        root_node.public_deps.add(root_node)
+        root_node.transitive_closure[root_node.name] = root_node
+        if profile_build:
+            root_node.conanfile.settings_build = profile_build.processed_settings.copy()
+            root_node.conanfile.settings_target = None
         dep_graph.add_node(root_node)
 
         # enter recursive computation
         t1 = time.time()
         self._expand_node(root_node, dep_graph, Requirements(), None, None, check_updates,
-                          update, remotes, profile_host, graph_lock)
+                          update, remotes, profile_host, profile_build, graph_lock)
         logger.debug("GRAPH: Time to load deps %s" % (time.time() - t1))
         return dep_graph
 
     def extend_build_requires(self, graph, node, build_requires_refs, check_updates, update,
-                              remotes, profile_host, graph_lock):
+                              remotes, profile_host, profile_build, graph_lock):
         # The options that will be defined in the node will be the real options values that have
         # been already propagated downstream from the dependency graph. This will override any
         # other possible option in the build_requires dependency graph. This means that in theory
@@ -78,29 +80,36 @@ class DepsGraphBuilder(object):
 
         conanfile = node.conanfile
         scope = conanfile.display_name
-        requires = [Requirement(ref) for ref in build_requires_refs]
-        for r in requires:
+
+        build_requires = []
+        for ref, context in build_requires_refs:
+            r = Requirement(ref)
             r.build_require = True
+            r.build_require_context = context
+            build_requires.append(r)
 
         if graph_lock:
             graph_lock.pre_lock_node(node)
-            graph_lock.lock_node(node, requires, build_requires=True)
+            # TODO: Add info about context?
+            graph_lock.lock_node(node, build_requires, build_requires=True)
 
-        self._resolve_ranges(graph, requires, scope, update, remotes)
+        self._resolve_ranges(graph, build_requires, scope, update, remotes)
 
-        for require in requires:
-            self._expand_require(require, node, graph, check_updates, update,
-                                 remotes, profile_host, new_reqs, new_options, graph_lock)
+        for br in build_requires:
+            context_switch = bool(br.build_require_context == CONTEXT_BUILD)
+            populate_settings_target = context_switch  # Avoid 'settings_target' for BR-host
+            self._expand_require(br, node, graph, check_updates, update,
+                                 remotes, profile_host, profile_build, new_reqs, new_options,
+                                 graph_lock, context_switch=context_switch,
+                                 populate_settings_target=populate_settings_target)
 
         new_nodes = set(n for n in graph.nodes if n.package_id is None)
         # This is to make sure that build_requires have precedence over the normal requires
-        ordered_closure = list(node.public_closure.items())
-        ordered_closure.sort(key=lambda x: x[1] not in new_nodes)
-        node.public_closure = OrderedDict(ordered_closure)
+        node.public_closure.sort(key_fn=lambda x: x not in new_nodes)
         return new_nodes
 
-    def _expand_node(self, node, graph, down_reqs, down_ref, down_options,
-                     check_updates, update, remotes, profile_host, graph_lock):
+    def _expand_node(self, node, graph, down_reqs, down_ref, down_options, check_updates, update,
+                     remotes, profile_host, profile_build, graph_lock):
         """ expands the dependencies of the node, recursively
 
         param node: Node object to be expanded in this step
@@ -116,11 +125,14 @@ class DepsGraphBuilder(object):
         for require in node.conanfile.requires.values():
             if require.override:
                 continue
-            self._expand_require(require, node, graph, check_updates, update,
-                                 remotes, profile_host, new_reqs, new_options, graph_lock)
+            self._expand_require(require, node, graph, check_updates, update, remotes, profile_host,
+                                 profile_build, new_reqs, new_options, graph_lock,
+                                 context_switch=False)
 
     def _resolve_ranges(self, graph, requires, consumer, update, remotes):
         for require in requires:
+            if require.locked_id:  # if it is locked, nothing to resolved
+                continue
             self._resolver.resolve(require, consumer, update, remotes)
         self._resolve_cached_alias(requires, graph)
 
@@ -146,55 +158,57 @@ class DepsGraphBuilder(object):
 
         if graph_lock:  # No need to evaluate, they are hardcoded in lockfile
             graph_lock.lock_node(node, node.conanfile.requires.values())
-            new_reqs = None
-        else:
-            # propagation of requirements only necessary if not locked
-            new_reqs = node.conanfile.requires.update(down_reqs, self._output, node.ref, down_ref)
-            # if there are version-ranges, resolve them before expanding each of the requirements
-            # Resolve possible version ranges of the current node requirements
-            # new_reqs is a shallow copy of what is propagated upstream, so changes done by the
-            # RangeResolver are also done in new_reqs, and then propagated!
-            conanfile = node.conanfile
-            scope = conanfile.display_name
-            self._resolve_ranges(graph, conanfile.requires.values(), scope, update, remotes)
 
-            if not hasattr(conanfile, "_conan_evaluated_requires"):
-                conanfile._conan_evaluated_requires = conanfile.requires.copy()
-            elif conanfile.requires != conanfile._conan_evaluated_requires:
-                raise ConanException("%s: Incompatible requirements obtained in different "
-                                     "evaluations of 'requirements'\n"
-                                     "    Previous requirements: %s\n"
-                                     "    New requirements: %s"
-                                     % (scope, list(conanfile._conan_evaluated_requires.values()),
-                                        list(conanfile.requires.values())))
+        # propagation of requirements can be necessary if some nodes are not locked
+        new_reqs = node.conanfile.requires.update(down_reqs, self._output, node.ref, down_ref)
+        # if there are version-ranges, resolve them before expanding each of the requirements
+        # Resolve possible version ranges of the current node requirements
+        # new_reqs is a shallow copy of what is propagated upstream, so changes done by the
+        # RangeResolver are also done in new_reqs, and then propagated!
+        conanfile = node.conanfile
+        scope = conanfile.display_name
+        self._resolve_ranges(graph, conanfile.requires.values(), scope, update, remotes)
+
+        if not hasattr(conanfile, "_conan_evaluated_requires"):
+            conanfile._conan_evaluated_requires = conanfile.requires.copy()
+        elif conanfile.requires != conanfile._conan_evaluated_requires:
+            raise ConanException("%s: Incompatible requirements obtained in different "
+                                 "evaluations of 'requirements'\n"
+                                 "    Previous requirements: %s\n"
+                                 "    New requirements: %s"
+                                 % (scope, list(conanfile._conan_evaluated_requires.values()),
+                                    list(conanfile.requires.values())))
 
         return new_options, new_reqs
 
-    def _expand_require(self, require, node, graph, check_updates, update,
-                        remotes, profile_host, new_reqs, new_options, graph_lock):
+    def _expand_require(self, require, node, graph, check_updates, update, remotes, profile_host,
+                        profile_build, new_reqs, new_options, graph_lock, context_switch,
+                        populate_settings_target=True):
         # Handle a requirement of a node. There are 2 possibilities
         #    node -(require)-> new_node (creates a new node in the graph)
         #    node -(require)-> previous (creates a diamond with a previously existing node)
 
         # If the required is found in the node ancestors a loop is being closed
-        # TODO: allow bootstrapping, use references instead of names
-        name = require.ref.name
-        if name in node.ancestors or name == node.name:
-            raise ConanException("Loop detected: '%s' requires '%s' which is an ancestor too"
-                                 % (node.ref, require.ref))
+        context = CONTEXT_BUILD if context_switch else node.context
+        name = require.ref.name  # TODO: allow bootstrapping, use references instead of names
+        if node.ancestors.get(name, context) or (name == node.name and context == node.context):
+            raise ConanException("Loop detected in context %s: '%s' requires '%s'"
+                                 " which is an ancestor too" % (context, node.ref, require.ref))
 
         # If the requirement is found in the node public dependencies, it is a diamond
-        previous = node.public_deps.get(name)
-        previous_closure = node.public_closure.get(name)
+        previous = node.public_deps.get(name, context=context)
+        previous_closure = node.public_closure.get(name, context=context)
         # build_requires and private will create a new node if it is not in the current closure
         if not previous or ((require.build_require or require.private) and not previous_closure):
             # new node, must be added and expanded (node -> new_node)
             new_node = self._create_new_node(node, graph, require, check_updates, update,
-                                             remotes, profile_host, graph_lock)
+                                             remotes, profile_host, profile_build, graph_lock,
+                                             context_switch=context_switch,
+                                             populate_settings_target=populate_settings_target)
 
             # The closure of a new node starts with just itself
-            new_node.public_closure = OrderedDict([(name, new_node)])
-            new_node.transitive_closure = OrderedDict([(name, new_node)])
+            new_node.public_closure.add(new_node)
+            new_node.transitive_closure[new_node.name] = new_node
             # The new created node is connected to the parent one
             node.connect_closure(new_node)
 
@@ -203,13 +217,13 @@ class DepsGraphBuilder(object):
                 # the new_node doesn't propagate downstream the "node" consumer, so its public_deps
                 # will be a copy of the node.public_closure, i.e. it can only cause conflicts in the
                 # new_node.public_closure.
-                new_node.public_deps = node.public_closure.copy()
-                new_node.public_deps[name] = new_node
+                new_node.public_deps.assign(node.public_closure)
+                new_node.public_deps.add(new_node)
             else:
-                node.transitive_closure[name] = new_node
+                node.transitive_closure[new_node.name] = new_node
                 # Normal requires propagate and can conflict with the parent "node.public_deps" too
-                new_node.public_deps = node.public_deps.copy()
-                new_node.public_deps[name] = new_node
+                new_node.public_deps.assign(node.public_deps)
+                new_node.public_deps.add(new_node)
 
                 # All the dependents of "node" are also connected now to "new_node"
                 for dep_node in node.inverse_closure:
@@ -217,58 +231,64 @@ class DepsGraphBuilder(object):
 
             # RECURSION, keep expanding (depth-first) the new node
             self._expand_node(new_node, graph, new_reqs, node.ref, new_options, check_updates,
-                              update, remotes, profile_host, graph_lock)
+                              update, remotes, profile_host, profile_build, graph_lock)
             if not require.private and not require.build_require:
-                node.transitive_closure.update(new_node.transitive_closure)
+                for name, n in new_node.transitive_closure.items():
+                    node.transitive_closure[name] = n
 
         else:  # a public node already exist with this name
             self._resolve_cached_alias([require], graph)
             # As we are closing a diamond, there can be conflicts. This will raise if conflicts
-            conflict = self._conflicting_references(previous.ref, require.ref, node.ref)
+            conflict = self._conflicting_references(previous, require.ref, node.ref)
             if conflict:  # It is possible to get conflict from alias, try to resolve it
                 self._resolve_recipe(node, graph, require, check_updates,
                                      update, remotes, profile_host, graph_lock)
                 # Maybe it was an ALIAS, so we can check conflict again
-                conflict = self._conflicting_references(previous.ref, require.ref, node.ref)
+                conflict = self._conflicting_references(previous, require.ref, node.ref)
                 if conflict:
                     raise ConanException(conflict)
 
             # Add current ancestors to the previous node and upstream deps
-            union = node.ancestors.union([node.name])
-            for n in previous.public_closure.values():
-                n.ancestors.update(union)
+            for n in previous.public_closure:
+                n.ancestors.add(node)
+                for item in node.ancestors:
+                    n.ancestors.add(item)
 
             node.connect_closure(previous)
             graph.add_edge(node, previous, require)
             if not require.private and not require.build_require:
-                node.transitive_closure.update(previous.transitive_closure)
+                for name, n in previous.transitive_closure.items():
+                    node.transitive_closure[name] = n
 
                 # All the upstream dependencies (public_closure) of the previously existing node
                 # now will be also connected to the node and to all its dependants
-                for name, n in previous.transitive_closure.items():
+                for n in previous.transitive_closure.values():
                     node.connect_closure(n)
                     for dep_node in node.inverse_closure:
                         dep_node.connect_closure(n)
 
             # Recursion is only necessary if the inputs conflict with the current "previous"
             # configuration of upstream versions and options
-            if not graph_lock and self._recurse(previous.public_closure, new_reqs, new_options):
+            if not graph_lock and self._recurse(previous.public_closure, new_reqs, new_options,
+                                                previous.context):
                 self._expand_node(previous, graph, new_reqs, node.ref, new_options, check_updates,
-                                  update, remotes, profile_host, graph_lock)
+                                  update, remotes, profile_host, profile_build, graph_lock)
 
     @staticmethod
-    def _conflicting_references(previous_ref, new_ref, consumer_ref=None):
-        if previous_ref.copy_clear_rev() != new_ref.copy_clear_rev():
+    def _conflicting_references(previous, new_ref, consumer_ref=None):
+        if previous.ref.copy_clear_rev() != new_ref.copy_clear_rev():
             if consumer_ref:
-                return ("Conflict in %s\n"
-                        "    Requirement %s conflicts with already defined %s\n"
-                        "    To change it, override it in your base requirements"
-                        % (consumer_ref, new_ref, previous_ref))
+                return ("Conflict in %s:\n"
+                        "    '%s' requires '%s' while '%s' requires '%s'.\n"
+                        "    To fix this conflict you need to override the package '%s' "
+                        "in your root package."
+                        % (consumer_ref, consumer_ref, new_ref, next(iter(previous.dependants)).src,
+                           previous.ref, new_ref.name))
             return True
         # Computed node, if is Editable, has revision=None
         # If new_ref.revision is None we cannot assume any conflict, the user hasn't specified
         # a revision, so it's ok any previous_ref
-        if previous_ref.revision and new_ref.revision and previous_ref.revision != new_ref.revision:
+        if previous.ref.revision and new_ref.revision and previous.ref.revision != new_ref.revision:
             if consumer_ref:
                 raise ConanException("Conflict in %s\n"
                                      "    Different revisions of %s has been requested"
@@ -276,16 +296,16 @@ class DepsGraphBuilder(object):
             return True
         return False
 
-    def _recurse(self, closure, new_reqs, new_options):
+    def _recurse(self, closure, new_reqs, new_options, context):
         """ For a given closure, if some requirements or options coming from downstream
         is incompatible with the current closure, then it is necessary to recurse
         then, incompatibilities will be raised as usually"""
         for req in new_reqs.values():
-            n = closure.get(req.ref.name)
-            if n and self._conflicting_references(n.ref, req.ref):
+            n = closure.get(req.ref.name, context=context)
+            if n and self._conflicting_references(n, req.ref):
                 return True
         for pkg_name, options_values in new_options.items():
-            n = closure.get(pkg_name)
+            n = closure.get(pkg_name, context=context)
             if n:
                 options = n.conanfile.options
                 for option, value in options_values.items():
@@ -301,27 +321,9 @@ class DepsGraphBuilder(object):
         """
         conanfile, ref = node.conanfile, node.ref
         try:
-            # Avoid extra time manipulating the sys.path for python
+            run_configure_method(conanfile, down_options, down_ref, ref)
+
             with get_env_context_manager(conanfile, without_python=True):
-                if hasattr(conanfile, "config"):
-                    if not ref:
-                        conanfile.output.warn("config() has been deprecated."
-                                              " Use config_options and configure")
-                    with conanfile_exception_formatter(str(conanfile), "config"):
-                        conanfile.config()
-                with conanfile_exception_formatter(str(conanfile), "config_options"):
-                    conanfile.config_options()
-                conanfile.options.propagate_upstream(down_options, down_ref, ref)
-                if hasattr(conanfile, "config"):
-                    with conanfile_exception_formatter(str(conanfile), "config"):
-                        conanfile.config()
-
-                with conanfile_exception_formatter(str(conanfile), "configure"):
-                    conanfile.configure()
-
-                conanfile.settings.validate()  # All has to be ok!
-                conanfile.options.validate()
-
                 # Update requirements (overwrites), computing new upstream
                 if hasattr(conanfile, "requirements"):
                     # If re-evaluating the recipe, in a diamond graph, with different options,
@@ -382,20 +384,42 @@ class DepsGraphBuilder(object):
         return new_ref, dep_conanfile, recipe_status, remote, locked_id
 
     def _create_new_node(self, current_node, dep_graph, requirement, check_updates,
-                         update, remotes, profile, graph_lock):
+                         update, remotes, profile_host, profile_build, graph_lock, context_switch,
+                         populate_settings_target):
+        # If there is a context_switch, it is because it is a BR-build
+        if context_switch:
+            profile = profile_build
+            context = CONTEXT_BUILD
+        else:
+            profile = profile_host if current_node.context == CONTEXT_HOST else profile_build
+            context = current_node.context
 
         result = self._resolve_recipe(current_node, dep_graph, requirement, check_updates, update,
                                       remotes, profile, graph_lock)
         new_ref, dep_conanfile, recipe_status, remote, locked_id = result
 
+        # Assign the profiles depending on the context
+        if profile_build:  # Keep existing behavior (and conanfile members) if no profile_build
+            dep_conanfile.settings_build = profile_build.processed_settings.copy()
+            if not context_switch:
+                if populate_settings_target:
+                    dep_conanfile.settings_target = current_node.conanfile.settings_target
+                else:
+                    dep_conanfile.settings_target = None
+            else:
+                if current_node.context == CONTEXT_HOST:
+                    dep_conanfile.settings_target = profile_host.processed_settings.copy()
+                else:
+                    dep_conanfile.settings_target = profile_build.processed_settings.copy()
+
         logger.debug("GRAPH: new_node: %s" % str(new_ref))
-        new_node = Node(new_ref, dep_conanfile)
+        new_node = Node(new_ref, dep_conanfile, context=context)
         new_node.revision_pinned = requirement.ref.revision is not None
         new_node.recipe = recipe_status
         new_node.remote = remote
         # Ancestors are a copy of the parent, plus the parent itself
-        new_node.ancestors = current_node.ancestors.copy()
-        new_node.ancestors.add(current_node.name)
+        new_node.ancestors.assign(current_node.ancestors)
+        new_node.ancestors.add(current_node)
 
         if locked_id is not None:
             new_node.id = locked_id
