@@ -1,10 +1,12 @@
 import os
+import platform
 import sys
 from contextlib import contextmanager
 
 from conans.client.run_environment import RunEnvironment
 from conans.client.tools.files import _path_equals, which
 from conans.errors import ConanException
+from conans.util.runners import check_output_runner
 
 
 @contextmanager
@@ -111,3 +113,45 @@ def remove_from_path(command):
 
     with environment_append({"PATH": curpath}):
         yield
+
+
+def env_diff(cmd, only_diff):
+    known_path_lists = ("include", "lib", "libpath", "path")
+    if platform.system() == "Windows":
+        cmd += " && set"
+    else:
+        cmd += " && export"
+    ret = check_output_runner(cmd)
+    new_env = {}
+    for line in ret.splitlines():
+        line = line.strip()
+
+        if line == "\n" or not line:
+            continue
+        try:
+            name_var, value = line.split("=", 1)
+            name_var = str(name_var)
+            value = str(value)
+            new_value = value.split(os.pathsep) if name_var.lower() in known_path_lists else value
+            # Return only new vars & changed ones, but only with the changed elements if the var is
+            # a list
+            if only_diff:
+                old_value = os.environ.get(name_var)
+                if name_var.lower() == "path":
+                    old_values_lower = [v.lower() for v in old_value.split(os.pathsep)]
+                    # Clean all repeated entries, not append if the element was already there
+                    new_env[name_var] = [v for v in new_value if v.lower() not in old_values_lower]
+                elif old_value and value.endswith(os.pathsep + old_value):
+                    # The new value ends with separator and the old value, is a list,
+                    # get only the new elements
+                    new_env[name_var] = value[:-(len(old_value) + 1)].split(os.pathsep)
+                elif value != old_value:
+                    # Only if the vcvars changed something, we return the variable,
+                    # otherwise is not vcvars related
+                    new_env[name_var] = new_value
+            else:
+                new_env[name_var] = new_value
+
+        except ValueError:
+            pass
+    return new_env
