@@ -1,9 +1,11 @@
-from conans.util.sha import sha1
-from conans.errors import ConanException
-import yaml
-import six
+
 import fnmatch
 
+import six
+import yaml
+
+from conans.errors import ConanException
+from conans.util.sha import sha1
 
 _falsey_options = ["false", "none", "0", "off", ""]
 
@@ -20,8 +22,8 @@ def option_not_exist_msg(option_name, existing_options):
     """ Someone is referencing an option that is not available in the current package
     options
     """
-    result = ["'options.%s' doesn't exist" % option_name]
-    result.append("Possible options are %s" % existing_options or "none")
+    result = ["option '%s' doesn't exist" % option_name,
+              "Possible options are %s" % existing_options or "none"]
     return "\n".join(result)
 
 
@@ -57,12 +59,27 @@ class PackageOptionValues(object):
     def __init__(self):
         self._dict = {}  # {option_name: PackageOptionValue}
         self._modified = {}
+        self._freeze = False
+
+    def __bool__(self):
+        return bool(self._dict)
+
+    def __contains__(self, key):
+        return key in self._dict
+
+    def __nonzero__(self):
+        return self.__bool__()
 
     def __getattr__(self, attr):
         if attr not in self._dict:
-            return None
+            raise ConanException(option_not_exist_msg(attr, list(self._dict.keys())))
         return self._dict[attr]
 
+    def __delattr__(self, attr):
+        if attr not in self._dict:
+            return
+        del self._dict[attr]
+
     def clear(self):
         self._dict.clear()
 
@@ -81,6 +98,9 @@ class PackageOptionValues(object):
     def fields(self):
         return sorted(list(self._dict.keys()))
 
+    def keys(self):
+        return self._dict.keys()
+
     def items(self):
         return sorted(list(self._dict.items()))
 
@@ -99,23 +119,30 @@ class PackageOptionValues(object):
     def remove(self, option_name):
         del self._dict[option_name]
 
-    def propagate_upstream(self, down_package_values, down_ref, own_ref, output, package_name):
+    def freeze(self):
+        self._freeze = True
+
+    def propagate_upstream(self, down_package_values, down_ref, own_ref, package_name):
         if not down_package_values:
             return
 
         assert isinstance(down_package_values, PackageOptionValues)
         for (name, value) in down_package_values.items():
-            current_value = self._dict.get(name)
-            if value == current_value:
+            if name in self._dict and self._dict.get(name) == value:
                 continue
 
+            if self._freeze:
+                raise ConanException("%s tried to change %s option %s to %s\n"
+                                     "but it was already defined as %s"
+                                     % (down_ref, own_ref, name, value, self._dict.get(name)))
+
             modified = self._modified.get(name)
             if modified is not None:
                 modified_value, modified_ref = modified
-                output.werror("%s tried to change %s option %s:%s to %s\n"
-                              "but it was already assigned to %s by %s"
-                              % (down_ref, own_ref, package_name, name, value,
-                                 modified_value, modified_ref))
+                raise ConanException("%s tried to change %s option %s:%s to %s\n"
+                                     "but it was already assigned to %s by %s"
+                                     % (down_ref, own_ref, package_name, name, value,
+                                        modified_value, modified_ref))
             else:
                 self._modified[name] = (value, down_ref)
                 self._dict[name] = value
@@ -148,14 +175,16 @@ class OptionsValues(object):
 
         # convert tuple "Pkg:option=value", "..." to list of tuples(name, value)
         if isinstance(values, tuple):
-            new_values = []
-            for v in values:
-                option, value = v.split("=")
-                new_values.append((option.strip(), value.strip()))
-            values = new_values
+            values = [item.split("=", 1) for item in values]
+
+        # convert dict {"Pkg:option": "value", "..": "..", ...} to list of tuples (name, value)
+        if isinstance(values, dict):
+            values = [(k, v) for k, v in values.items()]
 
         # handle list of tuples (name, value)
         for (k, v) in values:
+            k = k.strip()
+            v = v.strip() if isinstance(v, six.string_types) else v
             tokens = k.split(":")
             if len(tokens) == 2:
                 package, option = tokens
@@ -172,8 +201,9 @@ class OptionsValues(object):
             pkg_values.update(package_values)
 
     def scope_options(self, name):
-        self._reqs_options.setdefault(name, PackageOptionValues()).update(self._package_values)
-        self._package_values = PackageOptionValues()
+        if self._package_values:
+            self._reqs_options.setdefault(name, PackageOptionValues()).update(self._package_values)
+            self._package_values = PackageOptionValues()
 
     def descope_options(self, name):
         package_values = self._reqs_options.pop(name, None)
@@ -183,6 +213,9 @@ class OptionsValues(object):
     def clear_unscoped_options(self):
         self._package_values.clear()
 
+    def __contains__(self, item):
+        return item in self._package_values
+
     def __getitem__(self, item):
         return self._reqs_options.setdefault(item, PackageOptionValues())
 
@@ -216,6 +249,9 @@ class OptionsValues(object):
             return super(OptionsValues, self).__setattr__(attr, value)
         return setattr(self._package_values, attr, value)
 
+    def __delattr__(self, attr):
+        delattr(self._package_values, attr)
+
     def clear_indirect(self):
         for v in self._reqs_options.values():
             v.clear()
@@ -246,27 +282,19 @@ class OptionsValues(object):
         other_option=3
         OtherPack:opt3=12.1
         """
-        result = []
-        for line in text.splitlines():
-            line = line.strip()
-            if not line:
-                continue
-            name, value = line.split("=", 1)
-            result.append((name.strip(), value.strip()))
-        return OptionsValues(result)
+        options = tuple(line.strip() for line in text.splitlines() if line.strip())
+        return OptionsValues(options)
 
     @property
     def sha(self):
-        result = []
-        result.append(self._package_values.sha)
+        result = [self._package_values.sha]
         for key in sorted(list(self._reqs_options.keys())):
             result.append(self._reqs_options[key].sha)
         return sha1('\n'.join(result).encode())
 
     def serialize(self):
-        ret = {}
-        ret["options"] = self._package_values.serialize()
-        ret["req_options"] = {}
+        ret = {"options": self._package_values.serialize(),
+               "req_options": {}}
         for name, values in self._reqs_options.items():
             ret["req_options"][name] = values.serialize()
         return ret
@@ -285,6 +313,10 @@ class PackageOption(object):
         else:
             self._possible_values = sorted(str(v) for v in possible_values)
 
+    def copy(self):
+        result = PackageOption(self._possible_values, self._name)
+        return result
+
     def __bool__(self):
         if not self._value:
             return False
@@ -296,6 +328,9 @@ class PackageOption(object):
     def __str__(self):
         return str(self._value)
 
+    def __int__(self):
+        return int(self._value)
+
     def _check_option_value(self, value):
         """ checks that the provided value is allowed by current restrictions
         """
@@ -344,13 +379,22 @@ class PackageOptions(object):
         self._data = {str(k): PackageOption(v, str(k))
                       for k, v in definition.items()}
         self._modified = {}
+        self._freeze = False
+
+    def copy(self):
+        result = PackageOptions(None)
+        result._data = {k: v.copy() for k, v in self._data.items()}
+        return result
 
     def __contains__(self, option):
         return str(option) in self._data
 
     @staticmethod
     def loads(text):
-        return PackageOptions(yaml.load(text) or {})
+        return PackageOptions(yaml.safe_load(text) or {})
+
+    def get_safe(self, field, default=None):
+        return self._data.get(field, default)
 
     def validate(self):
         for child in self._data.values():
@@ -370,25 +414,25 @@ class PackageOptions(object):
     def clear(self):
         self._data = {}
 
-    def _check_field(self, field):
+    def _ensure_exists(self, field):
         if field not in self._data:
             raise ConanException(option_not_exist_msg(field, list(self._data.keys())))
 
     def __getattr__(self, field):
         assert field[0] != "_", "ERROR %s" % field
-        self._check_field(field)
+        self._ensure_exists(field)
         return self._data[field]
 
     def __delattr__(self, field):
         assert field[0] != "_", "ERROR %s" % field
-        self._check_field(field)
+        self._ensure_exists(field)
         del self._data[field]
 
     def __setattr__(self, field, value):
         if field[0] == "_" or field.startswith("values"):
             return super(PackageOptions, self).__setattr__(field, value)
 
-        self._check_field(field)
+        self._ensure_exists(field)
         self._data[field].value = value
 
     @property
@@ -414,43 +458,56 @@ class PackageOptions(object):
     def values(self, vals):
         assert isinstance(vals, PackageOptionValues)
         for (name, value) in vals.items():
-            self._check_field(name)
+            self._ensure_exists(name)
             self._data[name].value = value
 
-    def propagate_upstream(self, package_values, down_ref, own_ref, output, ignore_unknown=False):
-        """ ignore_unknown: do not raise Exception if the given option doesn't exist in this package.
-                            Useful for pattern defined options like "-o *:shared=True", for packages
-                            not defining the "shared" options, they will not fail
+    def initialize_patterns(self, values):
+        # Need to apply only those that exists
+        for option, value in values.items():
+            if option in self._data:
+                self._data[option].value = value
+
+    def freeze(self):
+        self._freeze = True
+
+    def propagate_upstream(self, package_values, down_ref, own_ref, pattern_options):
+        """
+        :param: package_values: PackageOptionValues({"shared": "True"}
+        :param: pattern_options: Keys from the "package_values" e.g. ["shared"] that shouldn't raise
+                                 if they are not existing options for the current object
         """
         if not package_values:
             return
 
         for (name, value) in package_values.items():
-            current_value = self._data.get(name)
-            if value == current_value:
+            if name in self._data and self._data.get(name) == value:
                 continue
 
+            if self._freeze:
+                raise ConanException("%s tried to change %s option %s to %s\n"
+                                     "but it was already defined as %s"
+                                     % (down_ref, own_ref, name, value, self._data.get(name)))
             modified = self._modified.get(name)
             if modified is not None:
                 modified_value, modified_ref = modified
-                output.werror("%s tried to change %s option %s to %s\n"
-                              "but it was already assigned to %s by %s"
-                              % (down_ref, own_ref, name, value, modified_value, modified_ref))
+                raise ConanException("%s tried to change %s option %s to %s\n"
+                                     "but it was already assigned to %s by %s"
+                                     % (down_ref, own_ref, name, value,
+                                        modified_value, modified_ref))
             else:
-                if ignore_unknown:
+                if name in pattern_options:  # If it is a pattern-matched option, should check field
                     if name in self._data:
                         self._data[name].value = value
                         self._modified[name] = (value, down_ref)
                 else:
-                    self._check_field(name)
+                    self._ensure_exists(name)
                     self._data[name].value = value
                     self._modified[name] = (value, down_ref)
 
 
 class Options(object):
-    """ all options of a package, both its own options and the upstream
-    ones.
-    Owned by conanfile
+    """ All options of a package, both its own options and the upstream ones.
+    Owned by ConanFile.
     """
     def __init__(self, options):
         assert isinstance(options, PackageOptions)
@@ -460,6 +517,17 @@ class Options(object):
         # are not public, not overridable
         self._deps_package_values = {}  # {name("Boost": PackageOptionValues}
 
+    def copy(self):
+        """ deepcopy, same as Settings"""
+        result = Options(self._package_options.copy())
+        result._deps_package_values = {k: v.copy() for k, v in self._deps_package_values.items()}
+        return result
+
+    def freeze(self):
+        self._package_options.freeze()
+        for v in self._deps_package_values.values():
+            v.freeze()
+
     @property
     def deps_package_values(self):
         return self._deps_package_values
@@ -503,35 +571,54 @@ class Options(object):
         for k, v in v._reqs_options.items():
             self._deps_package_values[k] = v.copy()
 
-    def propagate_upstream(self, down_package_values, down_ref, own_ref, output):
+    def propagate_upstream(self, down_package_values, down_ref, own_ref):
         """ used to propagate from downstream the options to the upper requirements
+        :param: down_package_values => {"*": PackageOptionValues({"shared": "True"})}
+        :param: down_ref
+        :param: own_ref: Reference of the current package => ConanFileReference
         """
         if not down_package_values:
             return
 
         assert isinstance(down_package_values, dict)
-        option_values = down_package_values.get(own_ref.name)
-        self._package_options.propagate_upstream(option_values, down_ref, own_ref, output, ignore_unknown=False)
-        if not option_values:
-            for package_pattern, package_option_values in down_package_values.items():
-                if own_ref.name != package_pattern and fnmatch.fnmatch(own_ref.name, package_pattern):
-                    self._package_options.propagate_upstream(package_option_values, down_ref, own_ref, output,
-                                                             ignore_unknown=True)
+        option_values = PackageOptionValues()
+        # First step is to accumulate all matching patterns, in sorted()=alphabetical order
+        # except the exact match
 
+        for package_pattern, package_option_values in sorted(down_package_values.items()):
+            if own_ref.name != package_pattern and fnmatch.fnmatch(own_ref.name, package_pattern):
+                option_values.update(package_option_values)
+        # These are pattern options, shouldn't raise if not existing
+        pattern_options = list(option_values.keys())
+        # Now, update with the exact match, that has higher priority
+        down_options = down_package_values.get(own_ref.name)
+        if down_options is not None:
+            option_values.update(down_options)
+
+        self._package_options.propagate_upstream(option_values, down_ref, own_ref,
+                                                 pattern_options=pattern_options)
+
+        # Upstream propagation to deps
         for name, option_values in sorted(list(down_package_values.items())):
             if name != own_ref.name:
                 pkg_values = self._deps_package_values.setdefault(name, PackageOptionValues())
-                pkg_values.propagate_upstream(option_values, down_ref, own_ref, output, name)
+                pkg_values.propagate_upstream(option_values, down_ref, own_ref, name)
 
-    def initialize_upstream(self, user_values):
+    def initialize_upstream(self, user_values, name=None):
         """ used to propagate from downstream the options to the upper requirements
         """
         if user_values is not None:
             assert isinstance(user_values, OptionsValues)
-            # This values setter implements an update, not an overwrite
+            # This code is necessary to process patterns like *:shared=True
+            # To apply to the current consumer, which might not have name
+            for pattern, pkg_options in sorted(user_values._reqs_options.items()):
+                if fnmatch.fnmatch(name or "", pattern):
+                    self._package_options.initialize_patterns(pkg_options)
+            # Then, the normal assignment of values, which could override patterns
             self._package_options.values = user_values._package_values
             for package_name, package_values in user_values._reqs_options.items():
-                pkg_values = self._deps_package_values.setdefault(package_name, PackageOptionValues())
+                pkg_values = self._deps_package_values.setdefault(package_name,
+                                                                  PackageOptionValues())
                 pkg_values.update(package_values)
 
     def validate(self):
@@ -543,10 +630,10 @@ class Options(object):
         for k, v in options._reqs_options.items():
             self._deps_package_values[k] = v.copy()
 
-    def clear_unused(self, references):
+    def clear_unused(self, prefs):
         """ remove all options not related to the passed references,
         that should be the upstream requirements
         """
-        existing_names = [r.conan.name for r in references]
+        existing_names = [pref.ref.name for pref in prefs]
         self._deps_package_values = {k: v for k, v in self._deps_package_values.items()
                                      if k in existing_names}
