@@ -1,42 +1,70 @@
 import copy
-from collections import OrderedDict
-from collections import defaultdict
+from collections import OrderedDict, defaultdict
 
+from conans.client import settings_preprocessor
+from conans.errors import ConanException
 from conans.model.env_info import EnvValues
 from conans.model.options import OptionsValues
-from conans.model.scope import Scopes, _root
 from conans.model.values import Values
 
 
 class Profile(object):
     """A profile contains a set of setting (with values), environment variables
-    and scopes"""
+    """
 
     def __init__(self):
-        # Sections
+        # Input sections, as defined by user profile files and command line
         self.settings = OrderedDict()
         self.package_settings = defaultdict(OrderedDict)
         self.env_values = EnvValues()
-        self.scopes = Scopes()
         self.options = OptionsValues()
-        self.build_requires = OrderedDict()  # conan_ref Pattern: list of conan_ref
+        self.build_requires = OrderedDict()  # ref pattern: list of ref
+
+        # Cached processed values
+        self.processed_settings = None  # Settings with values, and smart completion
+        self._user_options = None
+        self._package_settings_values = None
+        self.dev_reference = None  # Reference of the package being develop
 
     @property
-    def settings_values(self):
-        return Values.from_list(list(self.settings.items()))
+    def user_options(self):
+        if self._user_options is None:
+            self._user_options = self.options.copy()
+        return self._user_options
 
     @property
     def package_settings_values(self):
-        result = {}
-        for pkg, settings in self.package_settings.items():
-            result[pkg] = list(settings.items())
-        return result
+        if self._package_settings_values is None:
+            self._package_settings_values = {}
+            for pkg, settings in self.package_settings.items():
+                self._package_settings_values[pkg] = list(settings.items())
+        return self._package_settings_values
+
+    def process_settings(self, cache, preprocess=True):
+        assert self.processed_settings is None, "processed settings must be None"
+        self.processed_settings = cache.settings.copy()
+        self.processed_settings.values = Values.from_list(list(self.settings.items()))
+        if preprocess:
+            settings_preprocessor.preprocess(self.processed_settings)
+            # Redefine the profile settings values with the preprocessed ones
+            # FIXME: Simplify the values.as_list()
+            self.settings = OrderedDict(self.processed_settings.values.as_list())
+
+            # Preprocess also scoped settings
+            for pkg, pkg_settings in self.package_settings.items():
+                pkg_profile = Profile()
+                pkg_profile.settings = self.settings
+                pkg_profile.update_settings(pkg_settings)
+                try:
+                    pkg_profile.process_settings(cache=cache, preprocess=True)
+                except Exception as e:
+                    pkg_profile = ["{}={}".format(k, v) for k, v in pkg_profile.settings.items()]
+                    raise ConanException("Error in resulting settings for package"
+                                         " '{}': {}\n{}".format(pkg, e, '\n'.join(pkg_profile)))
+                # TODO: Assign the _validated_ settings and do not compute again
 
     def dumps(self):
-        result = ["[build_requires]"]
-        for pattern, req_list in self.build_requires.items():
-            result.append("%s: %s" % (pattern, ", ".join(str(r) for r in req_list)))
-        result.append("[settings]")
+        result = ["[settings]"]
         for name, value in self.settings.items():
             result.append("%s=%s" % (name, value))
         for package, values in self.package_settings.items():
@@ -46,12 +74,9 @@ class Profile(object):
         result.append("[options]")
         result.append(self.options.dumps())
 
-        result.append("[scopes]")
-        if self.scopes[_root].get("dev", None):
-            # FIXME: Ugly _root import
-            del self.scopes[_root]["dev"]  # Do not include dev
-        scopes_txt = self.scopes.dumps()
-        result.append(scopes_txt)
+        result.append("[build_requires]")
+        for pattern, req_list in self.build_requires.items():
+            result.append("%s: %s" % (pattern, ", ".join(str(r) for r in req_list)))
 
         result.append("[env]")
         result.append(self.env_values.dumps())
@@ -61,7 +86,6 @@ class Profile(object):
     def update(self, other):
         self.update_settings(other.settings)
         self.update_package_settings(other.package_settings)
-        self.update_scopes(other.scopes)
         # this is the opposite
         other.env_values.update(self.env_values)
         self.env_values = other.env_values
@@ -79,12 +103,13 @@ class Profile(object):
         res = copy.copy(self.settings)
         if new_settings:
             # Invalidate the current subsettings if the parent setting changes
-            # Example: new_settings declare a different "compiler", so invalidate the current "compiler.XXX"
+            # Example: new_settings declare a different "compiler",
+            # so invalidate the current "compiler.XXX"
             for name, value in new_settings.items():
                 if "." not in name:
                     if name in self.settings and self.settings[name] != value:
                         for cur_name, _ in self.settings.items():
-                            if cur_name.startswith(name):
+                            if cur_name.startswith("%s." % name):
                                 del res[cur_name]
             # Now merge the new values
             res.update(new_settings)
@@ -95,10 +120,3 @@ class Profile(object):
         Specified package settings are prioritized to profile"""
         for package_name, settings in package_settings.items():
             self.package_settings[package_name].update(settings)
-
-    def update_scopes(self, new_scopes):
-        """Mix the specified settings with the current profile.
-        Specified settings are prioritized to profile"""
-        # apply the current profile
-        if new_scopes:
-            self.scopes.update(new_scopes)
